@@ -64,6 +64,7 @@ def parse_options():
     parser.add_option("--fields", dest="fields", metavar="<FIELD>,<FIELD>...", default=None, type="string")
     parser.add_option("--clients", dest="clients", metavar="NUM", default=3, type="int")
     parser.add_option("-h", "--help", dest="help", default=False, action="store_true")
+    parser.add_option("--debug", dest="debug", default=False, action="store_true")
     (options, args) = parser.parse_args()
 
     # Check validity of arguments
@@ -131,6 +132,7 @@ def parse_options():
     res["clients"] = options.clients
 
     res["auth_key"] = options.auth_key
+    res["debug"] = options.debug
     return res
 
 def get_tables(host, port, auth_key, tables):
@@ -366,8 +368,13 @@ def run_clients(options, db_table_set):
         if error_queue.empty() and not interrupt_event.is_set():
             print_progress(1.0)
 
-        # Continue past the progress output line
+        # Continue past the progress output line and print total rows processed
+        def plural(num, text):
+            return "%d %s%s" % (num, text, "" if num == 1 else "s")
+
         print ""
+        print "%s exported from %s" % (plural(sum([info[0].value for info in progress_info]), "row"),
+                                       plural(len(db_table_set), "table"))
     finally:
         signal.signal(signal.SIGINT, signal.SIG_DFL)
 
@@ -378,8 +385,9 @@ def run_clients(options, db_table_set):
         # multiprocessing queues don't handling tracebacks, so they've already been stringified in the queue
         while not error_queue.empty():
             error = error_queue.get()
-            print >> sys.stderr, "Traceback: %s" % (error[2])
-            print >> sys.stderr, "%s: %s" % (error[0].__name__, error[1])
+            print >> sys.stderr, "%s" % (error[1])
+            if options["debug"]:
+                print >> sys.stderr, "%s traceback: %s" % (error[0].__name__, error[2])
             raise RuntimeError("Errors occurred during export")
 
 def main():
